@@ -28,7 +28,8 @@ macro_rules! define_mat3_struct {
         /// A 3x3 column major matrix.
         ///
         /// This 3x3 matrix type features convenience methods for creating and using linear and
-        /// affine transformations.
+        /// affine transformations. If you are primarily dealing with 2D affine transformations the
+        /// [`Affine2`] type is much faster and more space efficient than using a 3x3 matrix.
         ///
         /// Linear transformations including 3D rotation and scale can be created using methods
         /// such as [`Self::from_diagonal()`], [`Self::from_quat()`], [`Self::from_axis_angle()`],
@@ -197,6 +198,26 @@ macro_rules! impl_mat3_methods {
             Self(Matrix3x3::from_scale(scale.0))
         }
 
+        /// Creates a 3x3 matrix from the first 9 values in `slice`.
+        ///
+        /// # Panics
+        ///
+        /// Panics if `slice` is less than 9 elements long.
+        #[inline(always)]
+        pub fn from_cols_slice(slice: &[$t]) -> Self {
+            Self(Matrix3x3::from_cols_slice(slice))
+        }
+
+        /// Writes the columns of `self` to the first 9 elements in `slice`.
+        ///
+        /// # Panics
+        ///
+        /// Panics if `slice` is less than 9 elements long.
+        #[inline(always)]
+        pub fn write_cols_to_slice(self, slice: &mut [$t]) {
+            Matrix3x3::write_cols_to_slice(&self.0, slice)
+        }
+
         /// Returns the matrix column for the given `index`.
         ///
         /// # Panics
