@@ -27,6 +27,8 @@ macro_rules! impl_quat_tests {
 
             let a1: [$t; 4] = q1.into();
             assert_eq!([v1.x, v1.y, v1.z, v1.w], a1);
+
+            assert_eq!(q1, $quat::from_array(a1));
         }
 
         #[test]
@@ -307,11 +309,11 @@ macro_rules! impl_quat_tests {
         fn test_slice() {
             let a: [$t; 4] =
                 $quat::from_euler(EulerRot::YXZ, deg(30.0), deg(60.0), deg(90.0)).into();
-            let b = $quat::from_slice_unaligned(&a);
+            let b = $quat::from_slice(&a);
             let c: [$t; 4] = b.into();
             assert_eq!(a, c);
             let mut d = [0.0, 0.0, 0.0, 0.0];
-            b.write_to_slice_unaligned(&mut d[..]);
+            b.write_to_slice(&mut d[..]);
             assert_eq!(a, d);
         }
 
